@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2019 The Qt Company Ltd.
+** Copyright (C) 2020 The Qt Company Ltd.
 ** Copyright (C) 2016 by Southwest Research Institute (R)
 ** Contact: http://www.qt-project.org/legal
 **
@@ -45,65 +45,55 @@
 QT_BEGIN_NAMESPACE
 
 /*!
-    \headerfile <QFloat16>
-    \title 16-bit Floating Point Support
+    \class qfloat16
+    \keyword 16-bit Floating Point Support
     \ingroup funclists
-    \brief The <QFloat16> header file provides 16-bit floating point support.
+    \inmodule QtCore
+    \inheaderfile QFloat16
+    \brief Provides 16-bit floating point support.
 
-    This header file provides support for half-precision (16-bit) floating
-    point data with the class \c qfloat16.  It is fully compliant with IEEE
-    754 as a storage type.  This implies that any arithmetic operation on a
-    \c qfloat16 instance results in the value first being converted to a
-    \c float.  This conversion to and from \c float is performed by hardware
-    when possible, but on processors that do not natively support half-precision,
-    the conversion is performed through a sequence of lookup table operations.
+    The \c qfloat16 class provides support for half-precision (16-bit) floating
+    point data.  It is fully compliant with IEEE 754 as a storage type.  This
+    implies that any arithmetic operation on a \c qfloat16 instance results in
+    the value first being converted to a \c float.  This conversion to and from
+    \c float is performed by hardware when possible, but on processors that do
+    not natively support half-precision, the conversion is performed through a
+    sequence of lookup table operations.
 
     \c qfloat16 should be treated as if it were a POD (plain old data) type.
     Consequently, none of the supported operations need any elaboration beyond
     stating that it supports all arithmetic operators incident to floating point
     types.
 
+    \note On x86 and x86-64 that to get hardware accelerated conversions you must
+    compile with F16C or AVX2 enabled, or use qFloatToFloat16() and qFloatFromFloat16()
+    which will detect F16C at runtime.
+
     \since 5.9
 */
 
-/*!
-    \macro QT_NO_FLOAT16_OPERATORS
-    \relates <QFloat16>
-    \since 5.12.4
-
-    Defining this macro disables the arithmetic operators for qfloat16.
-
-    This is only necessary on Visual Studio 2017 (and earlier) when including
-    \c {<QFloat16>} and \c{<bitset>} in the same translation unit, which would
-    otherwise cause a compilation error due to a toolchain bug (see
-    [QTBUG-72073]).
-*/
-
 /*!
     \fn bool qIsInf(qfloat16 f)
-    \relates <QFloat16>
+    \relates qfloat16
+    \overload qIsInf(float)
 
     Returns true if the \c qfloat16 \a {f} is equivalent to infinity.
-
-    \sa qIsInf
 */
 
 /*!
     \fn bool qIsNaN(qfloat16 f)
-    \relates <QFloat16>
+    \relates qfloat16
+    \overload qIsNaN(float)
 
     Returns true if the \c qfloat16 \a {f} is not a number (NaN).
-
-    \sa qIsNaN
 */
 
 /*!
     \fn bool qIsFinite(qfloat16 f)
-    \relates <QFloat16>
+    \relates qfloat16
+    \overload qIsFinite(float)
 
     Returns true if the \c qfloat16 \a {f} is a finite number.
-
-    \sa qIsFinite
 */
 
 /*!
@@ -112,8 +102,6 @@ QT_BEGIN_NAMESPACE
     \fn bool qfloat16::isInf() const noexcept
 
     Tests whether this \c qfloat16 value is an infinity.
-
-    \sa qIsInf()
 */
 
 /*!
@@ -122,15 +110,13 @@ QT_BEGIN_NAMESPACE
     \fn bool qfloat16::isNaN() const noexcept
 
     Tests whether this \c qfloat16 value is "not a number".
-
-    \sa qIsNaN()
 */
 
 /*!
     \since 5.14
     \fn bool qfloat16::isNormal() const noexcept
 
-    Tests whether this \c qfloat16 value is finite and in normal form.
+    Returns \c true if this \c qfloat16 value is finite and in normal form.
 
     \sa qFpClassify()
 */
@@ -141,16 +127,29 @@ QT_BEGIN_NAMESPACE
     \fn bool qfloat16::isFinite() const noexcept
 
     Tests whether this \c qfloat16 value is finite.
+*/
 
-    \sa qIsFinite()
+/*!
+    \since 5.15
+    \fn qfloat16 qfloat16::copySign(qfloat16 sign) const noexcept
+
+    Returns a qfloat16 with the sign of \a sign but the rest of its value taken
+    from this qfloat16. Serves as qfloat16's equivalent of std::copysign().
+*/
+
+/*!
+    \fn int qFpClassify(qfloat16 val)
+    \relates qfloat16
+    \since 5.14
+    \overload qFpClassify(float)
+
+    Returns the floating-point class of \a val.
 */
 
 /*!
     \internal
     \since 5.14
     Implements qFpClassify() for qfloat16.
-
-    \sa qFpClassify()
 */
 int qfloat16::fpClassify() const noexcept
 {
@@ -159,23 +158,22 @@ int qfloat16::fpClassify() const noexcept
 }
 
 /*! \fn int qRound(qfloat16 value)
-    \relates <QFloat16>
+    \relates qfloat16
+    \overload qRound(float)
 
     Rounds \a value to the nearest integer.
-
-    \sa qRound
 */
 
 /*! \fn qint64 qRound64(qfloat16 value)
-    \relates <QFloat16>
+    \relates qfloat16
+    \overload qRound64(float)
 
     Rounds \a value to the nearest 64-bit integer.
-
-    \sa qRound64
 */
 
 /*! \fn bool qFuzzyCompare(qfloat16 p1, qfloat16 p2)
-    \relates <QFloat16>
+    \relates qfloat16
+    \overload qFuzzyCompare(float, float)
 
     Compares the floating point value \a p1 and \a p2 and
     returns \c true if they are considered equal, otherwise \c false.
@@ -248,10 +246,13 @@ static void qFloatFromFloat16_fast(float *, const quint16 *, qsizetype) noexcept
 #endif
 /*!
     \since 5.11
-    \relates <QFloat16>
+    \relates qfloat16
 
     Converts \a len floats from \a in to qfloat16 and stores them in \a out.
     Both \a in and \a out must have \a len allocated entries.
+
+    This function is faster than converting values one by one, and will do runtime
+    F16C detection on x86 and x86-64 hardware.
 */
 Q_CORE_EXPORT void qFloatToFloat16(qfloat16 *out, const float *in, qsizetype len) noexcept
 {
@@ -264,10 +265,13 @@ Q_CORE_EXPORT void qFloatToFloat16(qfloat16 *out, const float *in, qsizetype len
 
 /*!
     \since 5.11
-    \relates <QFloat16>
+    \relates qfloat16
 
     Converts \a len qfloat16 from \a in to floats and stores them in \a out.
     Both \a in and \a out must have \a len allocated entries.
+
+    This function is faster than converting values one by one, and will do runtime
+    F16C detection on x86 and x86-64 hardware.
 */
 Q_CORE_EXPORT void qFloatFromFloat16(float *out, const qfloat16 *in, qsizetype len) noexcept
 {
