@@ -45,6 +45,7 @@
 #include <private/qtextdocument_p.h>
 #include <qtextlayout.h>
 #include <qpointer.h>
+#include <qscopedvaluerollback.h>
 #include <qtextobject.h>
 #include <qtextcursor.h>
 #include <qdebug.h>
@@ -68,14 +69,14 @@ public:
     void reformatBlocks(int from, int charsRemoved, int charsAdded);
     void reformatBlock(const QTextBlock &block);
 
-    inline void rehighlight(QTextCursor &cursor, QTextCursor::MoveOperation operation) {
-        inReformatBlocks = true;
+    inline void rehighlight(QTextCursor &cursor, QTextCursor::MoveOperation operation)
+    {
+        QScopedValueRollback<bool> bg(inReformatBlocks, true);
         cursor.beginEditBlock();
         int from = cursor.position();
         cursor.movePosition(operation);
         reformatBlocks(from, 0, cursor.position() - from);
         cursor.endEditBlock();
-        inReformatBlocks = false;
     }
 
     inline void _q_delayedRehighlight() {
@@ -86,7 +87,7 @@ public:
     }
 
     void applyFormatChanges();
-    QVector<QTextCharFormat> formatChanges;
+    QList<QTextCharFormat> formatChanges;
     QTextBlock currentBlock;
     bool rehighlightPending;
     bool inReformatBlocks;
@@ -98,7 +99,7 @@ void QSyntaxHighlighterPrivate::applyFormatChanges()
 
     QTextLayout *layout = currentBlock.layout();
 
-    QVector<QTextLayout::FormatRange> ranges = layout->formats();
+    QList<QTextLayout::FormatRange> ranges = layout->formats();
 
     const int preeditAreaStart = layout->preeditAreaPosition();
     const int preeditAreaLength = layout->preeditAreaText().length();
@@ -172,7 +173,7 @@ void QSyntaxHighlighterPrivate::reformatBlocks(int from, int charsRemoved, int c
     if (lastBlock.isValid())
         endPosition = lastBlock.position() + lastBlock.length();
     else
-        endPosition = doc->docHandle()->length();
+        endPosition = QTextDocumentPrivate::get(doc)->length();
 
     bool forceHighlightOfNextBlock = false;
 
@@ -297,8 +298,8 @@ void QSyntaxHighlighterPrivate::reformatBlock(const QTextBlock &block)
 QSyntaxHighlighter::QSyntaxHighlighter(QObject *parent)
     : QObject(*new QSyntaxHighlighterPrivate, parent)
 {
-    if (parent->inherits("QTextEdit")) {
-        QTextDocument *doc = parent->property("document").value<QTextDocument *>();
+    if (parent && parent->inherits("QTextEdit")) {
+        QTextDocument *doc = qvariant_cast<QTextDocument *>(parent->property("document"));
         if (doc)
             setDocument(doc);
     }
@@ -320,7 +321,7 @@ QSyntaxHighlighter::QSyntaxHighlighter(QTextDocument *parent)
 */
 QSyntaxHighlighter::~QSyntaxHighlighter()
 {
-    setDocument(0);
+    setDocument(nullptr);
 }
 
 /*!
@@ -600,7 +601,7 @@ QTextBlockUserData *QSyntaxHighlighter::currentBlockUserData() const
 {
     Q_D(const QSyntaxHighlighter);
     if (!d->currentBlock.isValid())
-        return 0;
+        return nullptr;
 
     return d->currentBlock.userData();
 }
