@@ -217,7 +217,7 @@ QAbstractItemModelTester::QAbstractItemModelTester(QAbstractItemModel *model, Fa
 
     Q_D(QAbstractItemModelTester);
 
-    const auto &runAllTests = [d] { d->runAllTests(); };
+    auto runAllTests = [d] { d->runAllTests(); };
 
     connect(model, &QAbstractItemModel::columnsAboutToBeInserted,
             this, runAllTests);
@@ -288,6 +288,12 @@ QAbstractItemModelTester::FailureReportingMode QAbstractItemModelTester::failure
     return d->failureReportingMode;
 }
 
+bool QAbstractItemModelTester::verify(bool statement, const char *statementStr, const char *description, const char *file, int line)
+{
+    Q_D(QAbstractItemModelTester);
+    return d->verify(statement, statementStr, description, file, line);
+}
+
 QAbstractItemModelTesterPrivate::QAbstractItemModelTesterPrivate(QAbstractItemModel *model, QAbstractItemModelTester::FailureReportingMode failureReportingMode)
     : model(model),
       failureReportingMode(failureReportingMode),
@@ -432,7 +438,7 @@ void QAbstractItemModelTesterPrivate::parent()
     // when asked for the parent of an invalid index.
     MODELTESTER_VERIFY(!model->parent(QModelIndex()).isValid());
 
-    if (!model->hasChildren())
+    if (model->rowCount() == 0)
         return;
 
     // Column 0                | Column 1    |
@@ -443,11 +449,12 @@ void QAbstractItemModelTesterPrivate::parent()
     // Common error test #1, make sure that a top level index has a parent
     // that is a invalid QModelIndex.
     QModelIndex topIndex = model->index(0, 0, QModelIndex());
+    MODELTESTER_VERIFY(topIndex.isValid());
     MODELTESTER_VERIFY(!model->parent(topIndex).isValid());
 
     // Common error test #2, make sure that a second level index has a parent
     // that is the first level index.
-    if (model->hasChildren(topIndex)) {
+    if (model->rowCount(topIndex) > 0) {
         QModelIndex childIndex = model->index(0, 0, topIndex);
         MODELTESTER_VERIFY(childIndex.isValid());
         MODELTESTER_COMPARE(model->parent(childIndex), topIndex);
@@ -459,7 +466,7 @@ void QAbstractItemModelTesterPrivate::parent()
     if (model->hasIndex(0, 1)) {
         QModelIndex topIndex1 = model->index(0, 1, QModelIndex());
         MODELTESTER_VERIFY(topIndex1.isValid());
-        if (model->hasChildren(topIndex) && model->hasChildren(topIndex1)) {
+        if (model->rowCount(topIndex) > 0 && model->rowCount(topIndex1) > 0) {
             QModelIndex childIndex = model->index(0, 0, topIndex);
             MODELTESTER_VERIFY(childIndex.isValid());
             QModelIndex childIndex1 = model->index(0, 0, topIndex1);
@@ -577,7 +584,7 @@ void QAbstractItemModelTesterPrivate::checkChildren(const QModelIndex &parent, i
  */
 void QAbstractItemModelTesterPrivate::data()
 {
-    if (!model->hasChildren())
+    if (model->rowCount() == 0 || model->columnCount() == 0)
         return;
 
     MODELTESTER_VERIFY(model->index(0, 0).isValid());
@@ -605,7 +612,7 @@ void QAbstractItemModelTesterPrivate::data()
     // Check that the alignment is one we know about
     QVariant textAlignmentVariant = model->data(model->index(0, 0), Qt::TextAlignmentRole);
     if (textAlignmentVariant.isValid()) {
-        Qt::Alignment alignment = textAlignmentVariant.value<Qt::Alignment>();
+        Qt::Alignment alignment = qvariant_cast<Qt::Alignment>(textAlignmentVariant);
         MODELTESTER_COMPARE(alignment, (alignment & (Qt::AlignHorizontal_Mask | Qt::AlignVertical_Mask)));
     }
 
@@ -813,13 +820,27 @@ bool QAbstractItemModelTesterPrivate::compare(const T1 &t1, const T2 &t2,
         break;
 
     case QAbstractItemModelTester::FailureReportingMode::Warning:
-        if (!result)
-            qCWarning(lcModelTest, formatString, actual, QTest::toString(t1), expected, QTest::toString(t2), file, line);
+        if (!result) {
+            auto t1string = QTest::toString(t1);
+            auto t2string = QTest::toString(t2);
+            qCWarning(lcModelTest, formatString, actual, t1string ? t1string : "(nullptr)",
+                                                 expected, t2string ? t2string : "(nullptr)",
+                                                 file, line);
+            delete [] t1string;
+            delete [] t2string;
+        }
         break;
 
     case QAbstractItemModelTester::FailureReportingMode::Fatal:
-        if (!result)
-            qFatal(formatString, actual, QTest::toString(t1), expected, QTest::toString(t2), file, line);
+        if (!result) {
+            auto t1string = QTest::toString(t1);
+            auto t2string = QTest::toString(t2);
+            qFatal(formatString, actual, t1string ? t1string : "(nullptr)",
+                                 expected, t2string ? t2string : "(nullptr)",
+                                 file, line);
+            delete [] t1string;
+            delete [] t2string;
+        }
         break;
     }
 
